<script setup lang="ts">
import { computed, ref } from 'vue';

import { useVbenDrawer } from '@vben/common-ui';
import { $t } from '@vben/locales';
import { addFullName, cloneDeep, getPopupContainer, listToTree } from '@vben/utils';

import { useVbenForm } from '#/adapter/form';
import { listEquType } from '#/api/eims/equ-type';
import { addInventory, getInventory, updateInventory } from '#/api/eims/inventory';
import { getDeptTree, userList } from '#/api/system/user';
import { TreeSelectPanel } from '#/components/tree';

import { drawerSchema } from './data';

const emit = defineEmits<{ reload: [] }>();

const isUpdate = ref(false);
const title = computed(() => {
  return isUpdate.value ? $t('pages.common.edit') : $t('pages.common.add');
});

const [BasicForm, formApi] = useVbenForm({
  commonConfig: {
    formItemClass: 'col-span-2',
    componentProps: {
      class: 'w-full'
    },
    labelWidth: 120
  },
  schema: drawerSchema(),
  showDefaultActions: false,
  wrapperClass: 'grid-cols-2'
});

const [BasicDrawer, drawerApi] = useVbenDrawer({
  onCancel: handleCancel,
  onConfirm: handleConfirm,
  async onOpenChange(isOpen) {
    if (!isOpen) {
      return null;
    }
    drawerApi.drawerLoading(true);
    const { id } = drawerApi.getData() as { id?: number | string };
    isUpdate.value = !!id;
    // 初始化
    await setupDeptSelect();
    await setupEquTypeSelect();

    formApi.updateSchema([
      {
        dependencies: {
          show: () => !isUpdate.value,
          triggerFields: ['']
        },
        fieldName: 'equTypesList'
      },
      {
        dependencies: {
          show: () => !isUpdate.value,
          triggerFields: ['']
        },
        fieldName: 'equStatusList'
      }
    ]);
    // 更新 && 赋值
    if (isUpdate.value && id) {
      const record = await getInventory(id);
      await formApi.setValues(record);
      // 更新时不可修改盘点范围
      if (isUpdate.value && record.userDept) {
        await setupUserOptions(record.userDept);
      }
    }

    drawerApi.drawerLoading(false);
  }
});

/**
 * 加载设备类树
 */
const equTypeTree = ref();
async function setupEquTypeSelect() {
  const result = (await listEquType()) || [];
  // 构建组件需要数据
  const equArray = result.map((item) => {
    return {
      ...item, // 保留原有对象的所有属性
      id: item.equTypeId,
      label: item.typeName,
      weight: item.orderNum
    };
  });
  // support i18n
  equArray.forEach((item) => {
    item.typeName = $t(item.typeName);
  });
  const equTree = listToTree(equArray, { id: 'id', pid: 'parentId' });
  equTypeTree.value = equTree;
}

/**
 * 用户的加载
 */
async function setupUserOptions(deptId: any) {
  const params = { deptId };
  const userPageResult = await userList({
    pageNum: 1,
    pageSize: 500,
    ...params
  });
  const options = userPageResult.rows.map((item) => ({
    label: item.nickName || item.userName,
    value: item.userId
  }));
  // 筛选
  const filterOption = (input: string, option: any) => {
    return option.label.toLowerCase().includes(input.toLowerCase());
  };

  const placeholder = options.length > 0 ? '请选择' : '该部门下暂无用户';
  formApi.updateSchema([
    {
      componentProps: { options, placeholder, filterOption },
      fieldName: 'inventoryUser'
    }
  ]);
}

/**
 * 初始化部门选择
 */
async function setupDeptSelect() {
  // updateSchema
  const deptTree = await getDeptTree();
  // 选中后显示在输入框的值 即父节点 / 子节点
  addFullName(deptTree, 'label', ' / ');
  formApi.updateSchema([
    {
      componentProps: (formModel) => ({
        class: 'w-full',
        fieldNames: {
          key: 'id',
          value: 'id',
          children: 'children'
        },
        getPopupContainer,
        async onSelect(deptId: number | string) {
          /** 根据部门ID加载用户 */
          await setupUserOptions(deptId);
          /** 变化后需要重新选择用户 */
          formModel.inventoryUser = undefined;
        },
        placeholder: '请选择',
        showSearch: true,
        treeData: deptTree,
        treeDefaultExpandAll: true,
        treeLine: { showLeafIcon: false },
        // 筛选的字段
        treeNodeFilterProp: 'label',
        // 选中后显示在输入框的值
        treeNodeLabelProp: 'fullName'
      }),
      fieldName: 'userDept'
    }
  ]);
}
/**
 * 通过回调更新 无法通过v-model
 * @param value 菜单选择是否严格模式
 */
function handleCheckStrictlyChange(value: boolean) {
  formApi.setFieldValue('checkStrictly', value);
}

const equTypeSelectRef = ref();
async function handleConfirm() {
  try {
    drawerApi.drawerLoading(true);
    const { valid } = await formApi.validate();
    if (!valid) {
      return;
    }
    // 这个用于提交
    const equTypes = equTypeSelectRef.value?.[0]?.getCheckedKeys() ?? [];
    const data = cloneDeep(await formApi.getValues());
    data.equTypes = equTypes;
    await (isUpdate.value ? updateInventory(data) : addInventory(data));
    emit('reload');
    await handleCancel();
  } catch (error) {
    console.error(error);
  } finally {
    drawerApi.drawerLoading(false);
  }
}

async function handleCancel() {
  drawerApi.close();
  await formApi.resetForm();
}
</script>

<template>
  <BasicDrawer :close-on-click-modal="false" :title="title" class="w-[600px]">
    <BasicForm>
      <template #equTypesList="slotProps">
        <!-- check-strictly为readonly 不能通过v-model绑定 -->
        <TreeSelectPanel
          ref="equTypeSelectRef"
          :disabled="true"
          v-bind="slotProps"
          :check-strictly="formApi.form.values.checkStrictly"
          :tree-data="equTypeTree"
          @check-strictly-change="handleCheckStrictlyChange"
        />
      </template>
    </BasicForm>
  </BasicDrawer>
</template>
